'''
Created on Feb 25, 2016

@author: alevin
'''
from __future__ import absolute_import

import unittest
from pythesint.vocabulary import Vocabulary
from mock.mock import MagicMock


class VocabularyTest(unittest.TestCase):

    def setUp(self):
        self.cat = {'Category': 'Animal', 'Type': 'Cat'}
        self.dog = {'Category': 'Animal', 'Type': 'Dog'}
        self.mouse = {'Category': 'Animal', 'Type': 'Mouse'}
        self.house = {'Category': 'Construction', 'Type': 'house'}

    def test_find_keyword_get_list_not_implemented(self):
        vocab = Vocabulary('VOCAB MOCK')
        with self.assertRaises(NotImplementedError):
            vocab.find_keyword('an item')

    def test_find_keyword_not_found(self):
        vocab = Vocabulary('VOCAB MOCK')
        vocab.get_list = MagicMock(return_value=[self.cat, self.dog,
                                                 self.mouse])
        with self.assertRaises(IndexError):
            vocab.find_keyword('Horse')

    def test_find_keyword(self):
        vocab = Vocabulary('VOCAB MOCK')
        vocab.get_list = MagicMock(return_value=[self.cat, self.dog,
                                                 self.mouse, self.house])
        self.assertDictEqual(vocab.find_keyword('dog'), self.dog)

if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



from __future__ import absolute_import

import urllib2
from xml.dom.minidom import parse

from pythesint.json_vocabulary import JSONVocabulary

class CFVocabulary(JSONVocabulary):
    def _fetch_online_data(self):
        # Note the version number... Would probably be better to make it always
        # take the last version..
        u1 = urllib2.urlopen(self.url)
        dom = parse(u1)
        node = dom.childNodes[0] # should only contain the standard_name_table

        cf_list = []
        for entry in node.getElementsByTagName('entry'):
            standard_name = entry.attributes['id'].value
            units = ''
            if entry.getElementsByTagName(
                    'canonical_units')[0].childNodes:
                units = entry.getElementsByTagName(
                    'canonical_units')[0].childNodes[0].nodeValue
            grib = ''
            if entry.getElementsByTagName('grib')[0].childNodes:
                grib = entry.getElementsByTagName(
                    'grib')[0].childNodes[0].nodeValue
            amip = ''
            if entry.getElementsByTagName('amip')[0].childNodes:
                amip = entry.getElementsByTagName(
                    'amip')[0].childNodes[0].nodeValue
            desrc = ''
            if entry.getElementsByTagName('description')[0].childNodes:
                descr = entry.getElementsByTagName(
                    'description')[0].childNodes[0].nodeValue
            stdname = {
                    'standard_name': standard_name,
                    'canonical_units': units,
                    'grib': grib,
                    'amip': amip,
                    'description': descr
                }
            cf_list.append(stdname)
        return cf_list




from __future__ import absolute_import

from os import path
from pkg_resources import resource_string

import yaml

def get_list(name):
    ''' Get list of keywords from given vocabulary <name>
    '''
    return vocabularies[name].get_list()

def get_keyword(name, item):
    ''' Get keyword <item> from given vocabulary <name> '''
    return vocabularies[name].find_keyword(item)

def update_vocabulary(name):
    ''' Update vocabulary content '''
    vocabularies[name].update()

def update_all_vocabularies():
    ''' Update all vocabularies '''
    for name in vocabularies:
        update_vocabulary(name)

def _process_config():
    ''' Load info about all vocabularies from config and add to module '''
    # http://stackoverflow.com/questions/1621350/dynamically-adding-functions-to-a-python-module
    # http://stackoverflow.com/questions/4821104/python-dynamic-instantiation-from-string-name-of-a-class-in-dynamically-imported

    ## load config file
    config = yaml.load(resource_string(__name__, 'pythesintrc.yaml'))
    current_module = __import__(__name__)

    # add get_ and update_ functions
    for cnf in config:
        voc_module = __import__('pythesint.' + cnf['module'], fromlist=[__name__])
        voc_class = getattr(voc_module, cnf['class'])
        vocabulary = voc_class(cnf['name'], **cnf['kwargs'])
        vocabularies[cnf['name']] = vocabulary

        setattr(current_module, 'get_'+cnf['name'], vocabulary.find_keyword)
        setattr(current_module, 'get_'+cnf['name']+'_list', vocabulary.get_list)
        setattr(current_module, 'update_'+cnf['name'], vocabulary.update)

vocabularies = {}
_process_config()



class Vocabulary(object):
    def __init__(self, name, **kwargs):
        self.name = name
        for key in kwargs:
            setattr(self, key, kwargs[key])

    def find_keyword(self, item):
        ''' Return the dictionary containing the given item in the provided list.
        The function returns only the lowest level match, i.e., the
        dictionary where the subgroups of the matching group are empty or the only
        match, if there is just one.

        Examples:

        1) Searching "Active Remote Sensing" in the instruments list will return
            {
                'Category': 'Earth Remote Sensing Instruments',
                'Class': 'Active Remote Sensing',
                'Type': '',
                'Subtype': '',
                'Short_Name': '',
                'Long_Name': ''
            }

        2) Searching "ASAR" in the instruments list will return
            {
                'Category': 'Earth Remote Sensing Instruments',
                'Class': 'Active Remote Sensing',
                'Type': 'Imaging Radars',
                'Subtype': '',
                'Short_Name': 'ASAR',
                'Long_Name': 'Advanced Synthetic Aperature Radar'
            }

        3) Searching "surface_backwards_scattering_coefficient_of_radar_wave" will
        return

        DOCTESTS:

        '''
        matches = []
        matching_key = ''
        for d in self.get_list():
            for key in d.keys():
                if d[key].upper() == item.upper():
                    matches.append(d)
                    matching_key = key
                    break

        if len(matches) == 0:
            raise IndexError('%s is not found in %s!' % (item, self.name))

        keys = matches[0].keys()
        kw_group_index = keys.index(matching_key)
        ii = range(kw_group_index+1, len(keys))
        if len(matches) == 1:
            return matches[0]
        # OBS: This works for the gcmd keywords but makes no sense for the cf
        # standard names - therefore always search the cf standard names by
        # standard_name only..
        for m in matches:
            remaining = {}
            for i in ii:
                remaining[keys[i]] = m[keys[i]]
            if not any(val for val in remaining.itervalues()):
                return m

    def update(self):
        pass

    def get_list(self):
        raise NotImplementedError




from __future__ import absolute_import

from pythesint.json_vocabulary import  Vocabulary

'''
This module contains the ISO Topic Category list as listed at
http://gcmd.gsfc.nasa.gov/add/difguide/iso_topic_category.html

It seem to not be (freely) available in the ISO 19115 - Geographic Information
Metadata (http://www.isotc211.org/) Topic Category Code List
'''

class ISO19115Vocabulary(Vocabulary):
    iso_topic_category_list = [
        'Farming',
        'Biota',
        'Boundaries',
        'Climatology/Meteorology/Atmosphere',
        'Economy',
        'Elevation',
        'Environment',
        'Geoscientific Information',
        'Health',
        'Imagery/Base Maps/Earth Cover',
        'Intelligence/Military',
        'Inland Waters',
        'Location',
        'Oceans',
        'Planning Cadastre',
        'Society',
        'Structure',
        'Transportation',
        'Utilities/Communications',
        ]

    def get_list(self):
        ''' Convert list of keywords into list of dicts '''
        return [{self.category : keyword} for keyword in
            self.iso_topic_category_list]



from __future__ import absolute_import

import os
import json
from pkg_resources import resource_filename

from pythesint.vocabulary import Vocabulary

class JSONVocabulary(Vocabulary):
    def json_filename(self):
        json_path = resource_filename('pythesint', 'json')
        return os.path.join(json_path, '%s_list.json' % self.name.lower())

    def get_list(self):
        ''' Read list from JSON '''
        if not os.path.exists(self.json_filename()):
            self.update()
        return json.load(open(self.json_filename()))

    def update(self):
        ''' Write vocabulary to a JSON file '''
        print('Downloading and writing json file for %s' % self.name)
        json_path = os.path.split(self.json_filename())[0]
        if not os.path.exists(json_path):
            os.makedirs(json_path)
        with open(self.json_filename(), 'w') as out:
            json.dump(self._fetch_online_data(), out, indent=4)



from __future__ import absolute_import

import unittest
import os, json, shutil
from pkg_resources import resource_filename

import pythesint as pti

class PythesintTest(unittest.TestCase):

    # Missing tests
    #def test_write_json
    #def test_write_json_to_path

    def test_get_list(self):
        self.assertIsInstance(pti.get_wkv_variable_list(), list)
        self.assertIsInstance(pti.get_gcmd_instrument_list(), list)
        self.assertIsInstance(pti.get_gcmd_science_keyword_list(), list)
        self.assertIsInstance(pti.get_gcmd_provider_list(), list)
        self.assertIsInstance(pti.get_gcmd_platform_list(), list)
        self.assertIsInstance(pti.get_gcmd_location_list(), list)
        self.assertIsInstance(pti.get_gcmd_horizontalresolutionrange_list(), list)
        self.assertIsInstance(pti.get_gcmd_verticalresolutionrange_list(), list)
        self.assertIsInstance(pti.get_gcmd_temporalresolutionrange_list(), list)
        self.assertIsInstance(pti.get_gcmd_project_list(), list)
        self.assertIsInstance(pti.get_gcmd_rucontenttype_list(), list)
        self.assertIsInstance(pti.get_cf_standard_name_list(), list)
        self.assertIsInstance(pti.get_iso19115_topic_category_list(), list)
#        self.assertIsInstance(pti.get__list(), list)

    def test_remove_and_get_gcmd_instrument(self):
        if os.path.exists(resource_filename('pythesint', 'json')):
            shutil.rmtree(resource_filename('pythesint', 'json'))
        item = 'MERIS'
        self.assertIsInstance(pti.get_gcmd_instrument(item), dict)

    def test_get_gcmd_instrument(self):
        item = 'MERIS'
        self.assertIsInstance(pti.get_gcmd_instrument(item), dict)

    def test_get_gcmd_science_keyword(self):
        item = 'sigma naught'
        self.assertIsInstance(pti.get_gcmd_science_keyword(item), dict)

    def test_get_gcmd_provider(self):
        item = 'NERSC'
        self.assertIsInstance(pti.get_gcmd_provider(item), dict)

    def test_get_gcmd_platform(self):
        item = 'AQUA'
        self.assertIsInstance(pti.get_gcmd_platform(item), dict)

    def test_get_gcmd_location(self):
        item = 'NORWEGIAN SEA'
        self.assertIsInstance(pti.get_gcmd_location(item), dict)

    def test_get_gcmd_horizontalresolutionrange(self):
        item = '< 1 meter'
        self.assertIsInstance(pti.get_gcmd_horizontalresolutionrange(item), dict)

    def test_get_gcmd_verticalresolutionrange(self):
        item = '< 1 meter'
        self.assertIsInstance(pti.get_gcmd_verticalresolutionrange(item), dict)

    def test_get_gcmd_temporalresolutionrange(self):
        item = 'Decadal'
        self.assertIsInstance(pti.get_gcmd_temporalresolutionrange(item), dict)

    def test_get_gcmd_project(self):
        item = 'IPCC'
        self.assertIsInstance(pti.get_gcmd_project(item), dict)

    def test_get_gcmd_rucontenttype(self):
        item = 'THREDDS DATA'
        self.assertIsInstance(pti.get_gcmd_rucontenttype(item), dict)

    def test_get_cf_standard_name(self):
        item = 'surface_backwards_scattering_coefficient_of_radar_wave'
        self.assertIsInstance(pti.get_cf_standard_name(item), dict)

    def test_get_wkv_variable(self):
        item = 'surface_backwards_doppler_frequency_shift_of_radar_wave_due_to_surface_velocity'
        self.assertIsInstance(pti.get_wkv_variable(item), dict)

    def test_get_wkv_latitude(self):
        item = 'latitude'
        self.assertIsInstance(pti.get_wkv_variable(item), dict)

    def test_get_iso19115_topic_category(self):
        item = 'Oceans'
        self.assertIsInstance(pti.get_iso19115_topic_category(item), dict)

    def test_get_fake_instrument(self):
        item = 'FakeItem'
        self.assertRaises(IndexError, pti.get_gcmd_instrument, item)

    def test_update(self):
        pti.update_wkv_variable()
        pti.update_gcmd_instrument()
        pti.update_gcmd_science_keyword()
        pti.update_gcmd_provider()
        pti.update_gcmd_platform()
        pti.update_gcmd_location()
        pti.update_gcmd_horizontalresolutionrange()
        pti.update_gcmd_verticalresolutionrange()
        pti.update_gcmd_temporalresolutionrange()
        pti.update_gcmd_project()
        pti.update_gcmd_rucontenttype()
        pti.update_cf_standard_name()
        pti.update_iso19115_topic_category()

if __name__ == "__main__":
    unittest.main()



from __future__ import absolute_import

import urllib2
from collections import OrderedDict

from pythesint.json_vocabulary import  JSONVocabulary

class GCMDVocabulary(JSONVocabulary):
    def _fetch_online_data(self):
        ''' Return list of GCMD standard keywords at provided url

        Parameters
        ----------
        list_name : the GCMD list name (must be one of the items in the standard_lists
        dictionary) in which;
            url is the URL of the desired GCMD list of valid keywords
            keyword_groups is a list containing the grouping of the keywords, e.g.,
            ['Category', 'Short_Name', 'Long_Name'] - used for verification
        '''
        response = urllib2.urlopen(self.url)
        # Boolean to determine if line information in the response object should be
        # stored as keywords
        do_record = False

        gcmd_list = []
        for line in response.readlines():
            if 'Keyword Version' and 'Revision' in line:
                meta = line.split('","')
                gcmd_list.append({'Revision': meta[1][10:]})
            if do_record:
                gcmd_keywords = line.split('","')
                gcmd_keywords[0] = gcmd_keywords[0].strip('"')
                if gcmd_keywords[0] == 'NOT APPLICABLE':
                    continue
                # Remove last item (the ID is not needed)
                gcmd_keywords.pop(-1)
                if len(gcmd_keywords)!=len(kw_groups):
                    continue
                line_kw = OrderedDict()
                for i, key in enumerate(kw_groups):
                    line_kw[key] = gcmd_keywords[i]
                gcmd_list.append(line_kw)
            if line.split(',')[0].lower() == self.categories[0].lower():
                do_record = True
                kw_groups = line.split(',')
                kw_groups.pop(-1)
                # Make sure the group items are as expected
                if kw_groups != self.categories:
                    raise TypeError('%s is not equal to %s' % (kw_groups, self.categories))

        return gcmd_list



from __future__ import absolute_import

import urllib2

import yaml

from pythesint.json_vocabulary import JSONVocabulary

class WKVVocabulary(JSONVocabulary):
    def _fetch_online_data(self):
        ''' Return list of Well Known Variables from Nansat        '''
        response = urllib2.urlopen(self.url)
        return yaml.load(response.read())

